/*
 * UNICADO - UNIversity Conceptual Aircraft Design and Optimization
 *
 * Copyright (C) 2025 UNICADO consortium
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * Description:
 * This file is part of UNICADO.
 */

#include "standardSystemsDesign.h"
#include "../toolinfo.h"
#include <vector>

void standardSystemsDesign::setHtmlBody() {
      std::string plot_path = relativePath(rtIO_->getPlotDir(), rtIO_->getReportDir()) + "/" + data_->configuration.theProgramName;
      /* Add box data */
      report_.htmlReportStream() << "<div class=\"box data\">\n";
      /* Add headline Data*/
      report_.htmlReportStream() << "<caption>Program Mode: </caption>";
      if (data_->configuration.designCase == true) {
         report_.htmlReportStream() << "Sizing Mode" << std::endl;
      } else {
         report_.htmlReportStream() << "Analysis Mode" << std::endl;
      }
      report_.htmlReportStream() << "<h2>Systems Masses</h2>" << std::endl;
      if (data_->configuration.designCase == true) {
         report_.htmlReportStream() << "<div id=\"leftCol\">" << std::endl
                                    << "<table>" << std::endl
                                    << "<tr>" << std::endl
                                    << "<td><h3>Systems Group</h3></td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    if (data_->Systems.systemExists("ATA-21")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-21</td><td>Air Conditioning</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-21"))->Mass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-24")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-24</td><td>Electric System</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-24"))->Mass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-26")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-26</td><td>Fire Protection</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-26"))->Mass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-27")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-27</td><td>Flight Control</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-27"))->Mass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-29")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-29</td><td>Hydraulic System</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-29"))->Mass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-30")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-30</td><td>Anti-Icing</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-30"))->Mass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-XX")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-XX</td><td>Remaining Consumers (including Avionics)</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-XX"))->Mass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl
                                    << "<td>ATA-49</td><td>APU</td><td>";
                                    }
                                    if (data_->Systems.systemExists("ATA-49")) {
                                    report_.htmlReportStream() << std::setprecision(4)
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-49"))->Mass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
         report_.htmlReportStream()<< "<td><h2>Power Unit Group</h2></td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    if (data_->Systems.systemExists("ATA-28")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-28</td><td>Fuel System</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-28"))->Mass << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-36")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-36</td><td>Bleed Air</td><td>"
                                    << data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-36"))->Mass << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    report_.htmlReportStream() << "<td><h2>Furnishing Group</h2></td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    if (data_->Systems.systemExists("ATA-25")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-25</td><td>Furnishing</td><td>" <<
                                       data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-25"))->Mass << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-33")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-33</td><td>Lighting</td><td>" <<
                                       data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-33"))->Mass << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    if (data_->Systems.systemExists("ATA-38")) {
                                    report_.htmlReportStream() << std::setprecision(4) << "<td>ATA-38</td><td>Water & Waste</td><td>" <<
                                       data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-38"))->Mass << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl;
                                    }
                                    report_.htmlReportStream() << "<td><h2>Operator Items</h2></td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl
                                    << "<td>---</td><td>Op. items calc. in systemsDesign (liquid)</td><td>"
                                    << data_->data.Mass.waterAndChemicalsMass + data_->data.Mass.residualFuelMass
                                    << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl
                                    << "<td></td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "<tr>" << std::endl
                                    << "<td>---</td><td>not Alloc</td><td>" << data_->data.Mass.notAllocatedSystems << " kg</td>" << std::endl
                                    << "</tr>" << std::endl
                                    << "</table>" << std::endl
                                    << "</div>" << std::endl
                                    << "<div id=\"rightCol\">" << std::endl
         //Graphic:
                                    << "<img class=\"image-plot\" src=\"" + plot_path + "_systems_mass_plot.svg\">" << std::endl
                                    << "</div>" << std::endl;
      } else {
         report_.htmlReportStream() << "<td>The program was run in analysis mode..</td>" << std::endl
                                    << "<td>The masses were not redetermined!</td>" << std::endl;
      }
/* Mission Profile Section */
      report_.htmlReportStream() << "<!-- CeRAS Report Systems -->" << std::endl
                                 << "<!-- ################################################################################################### -->" << std::endl
                                 << "<div id=\"wrapper\">" << std::endl
                                 << "<div id=\"leftCol\">" << std::endl
//    std::string missionFile(myAcftPt->myMissionPt->missionFile.substr(myAcftPt->myMissionPt->missionFile.find_last_of('\\') + 1))
//    report << "<h1>Mission: " << missionFile << "</h1>" << std::endl
                                 << "<h2>Required Power during the Mission</h2>" << std::endl
                                 << "<font size=\"-1\">Values at the end of the mission step are given in each case.</font>" << std::endl
                                 << "<img class=\"image-plot\" src=\"" + plot_path + "_mission_steps_plot.svg\">" << std::endl
                                 << "<img class=\"image-plot\" src=\"" + plot_path + "_accumulated_power_plot.svg\">" << std::endl
                                 << "</div>" << std::endl
                                 << "<div id=\"rightCol\">" << std::endl
/* Off-takes from energy sources */
                                 << "<h2>Power Off-takes</h2>" << std::endl;
      for (size_t n = 0; n < data_->Systems.energySource.size(); n++) {
      report_.htmlReportStream() << "<img class=\"image-plot\" src=\"" + plot_path +
                                    "_mission_power_" + data_->Systems.energySource.at(n)->ATA + "_plot.svg\">" << std::endl;
      }
      report_.htmlReportStream() << "<font size=\"-1\">APU is sized (only) for ground operations at gate (not visible in this mission).</font>" << std::endl
                                 << "</div>\n"
                                 << "</div>" << std::endl
/* Mission power of energy conductors */
                                 << "<h2>Energy Conductors</h2>" << std::endl;
      for (size_t n = 0; n < data_->Systems.energyConductor.size(); n++) {
         report_.htmlReportStream() << "<img class=\"ceras2Col\" src=\"" + plot_path +
                                 "_mission_power_" + data_->Systems.energyConductor.at(n)->ATA + "_plot.svg\">" << std::endl;
      }
/* Mission power of energy sinks */
      report_.htmlReportStream() << "<h2>Energy Sinks</h2>" << std::endl;
      for (size_t n = 0; n < data_->Systems.energySink.size(); n++) {
      report_.htmlReportStream() << "<img class=\"ceras2Col\" src=\"" + plot_path +
                                    "_mission_power_" + data_->Systems.energySink.at(n)->ATA + "_plot.svg\">" << std::endl;
      }
/* Design Power Section */
      report_.htmlReportStream() << "<h2>Design Power: </h2>" << std::endl
                                 //<< "<div class=\"box plot\">\n"
                                 << "<img class=\"ceras2Col\" src=\"" + plot_path +
                                    "_energy_sinks_max_energy_plot.svg\">" << std::endl
                                 << "</div>\n"
                                 << "<!-- ################################################################################################### -->" << std::endl;
}

void standardSystemsDesign::setTexBody() {
    if (data_->configuration.designCase == true) {
        report_.texReportStream() << "\\section{Systems Design}" << std::endl;
    }
    if (data_->configuration.designCase && !data_->configuration.texReportWithoutMasses) {
        report_.texReportStream() << "\\begin{frame}" << std::endl
                                   << " \\frametitle{Systems Masses}" << std::endl
                                   << std::endl
                                   << std::endl
                                   << " \\begin{table}" << std::endl
                                   << " \\centering" << std::endl
                                   << " \\footnotesize" << std::endl
                                   << "     \\begin{tabularx}{260pt}{llrr}" << std::endl
                                   << "        \\toprule" << std::endl
                                   << "        \\textcolor{ilrdark}{System} & \\textcolor{ilrdark}{ATA No.} & \\textcolor{ilrdark}{Value} &\\textcolor{ilrdark}{Unit} \\\\"
                                      << std::endl
                                   << "        \\midrule[0.5pt]" << std::endl
                                             //\\textcolor{ilrdark}{$\\blacktriangleright$}
                                   << "          Air Conditioning & ATA-21 & "
                                      << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-21"))->Mass, 0)
                                      << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Electric System & ATA-24 & "
                                      << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-24"))->Mass, 0)
                                      << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Furnishing & ATA-25 & " << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-25"))->Mass, 0)
                                      << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Fire Protection & ATA-26 & "
                                      << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-26"))->Mass, 0)
                                      << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Fuel System & ATA-28 & "
                                      << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-28"))->Mass, 0) << "& $\\kilogram$\\\\" << std::endl
                                   << "          Hydraulic System & ATA-29 & "
                                      << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-29"))->Mass, 0)
                                      << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Anti Icing & ATA-30 & "
                                      << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-30"))->Mass, 0)
                                      << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Lighting & ATA-33 & " << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-33"))->Mass, 0)
                                      << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Bleed Air & ATA-36 & " << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-36"))->Mass, 0)
                                   << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Auxiliary Power Unit (APU) & ATA-49 & "
                                   << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-49"))->Mass, 0)
                                   << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Remaining consumers & ATA-XX & "
                                   << Rounding(data_->Systems.allSystems.at(data_->Systems.getSpecificSystem("ATA-XX"))->Mass, 0)
                                   << "& $\\kilogram$ \\\\" << std::endl
                                   << "          Not Allocated Mass &  & " << Rounding(data_->data.Mass.notAllocatedSystems, 0) << "& $\\kilogram$ \\\\" << std::endl
                                   << "        \\bottomrule" << std::endl
                                   << "     \\end{tabularx}" << std::endl
                                   << " \\end{table}" << std::endl
                                   << "\\end{frame}" << std::endl
                                   << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                                   << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                                   << "\\begin{frame}" << std::endl
                                   << " \\frametitle{Systems Masses}" << std::endl
                                   << std::endl
                                   << std::endl
                                   << " \\begin{figure}" << std::endl
                                   << "     \\tiny" << std::endl
                                   << "     \\def\\svgwidth{10cm}" << std::endl
                                   << "     \\import{" + win2lin(relativePath(rtIO_->getPlotDir(), rtIO_->getIODir()))
                                              + "}{" + data_->configuration.theProgramName + "_SystemsMass_plot.pdf_tex}" << std::endl
                                   << "     \\caption{\\scriptsize{ATA Systems Masses}}" << std::endl
                                   << " \\end{figure}" << std::endl
                                   << "\\end{frame}" << std::endl
                                   << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                                   << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                                   << std::endl;
    }
    report_.texReportStream() << "\\begin{frame}" << std::endl
                               << " \\frametitle{Mission Increments for Systems Off-takes}" << std::endl
                               << "     \\begin{figure}" << std::endl
                               << "     \\tiny" << std::endl
                               << "     \\def\\svgwidth{7cm}" << std::endl
                               << "     \\import{" +
                                                 win2lin(relativePath(rtIO_->getPlotDir(), rtIO_->getReportDir())) + "}{" +
                                                 data_->configuration.theProgramName + "_MissionSteps_plot.pdf_tex}" << std::endl
                               << "         \\caption{\\scriptsize{Mission increments and flight conditions}}" << std::endl
                               << "     \\end{figure}" << std::endl
                               << std::endl
                               << "\\scriptsize{Values at the end of the mission step are given in each case.}" << std::endl
                               << std::endl
                               << "\\end{frame}" << std::endl
                               << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                               << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                               << std::endl
                               << "\\begin{frame}" << std::endl
                               << " \\frametitle{Energy Sources}" << std::endl
                               << "     \\begin{figure}" << std::endl
                               << "         \\tiny" << std::endl
                               << "         \\def\\svgwidth{7cm}" << std::endl
                               << "         \\import{" + win2lin(relativePath(rtIO_->getPlotDir(), rtIO_->getReportDir())) + "}{"
                                                       + data_->configuration.theProgramName + "_missionPower_EnergySources_plot.pdf_tex}" << std::endl
                               << "         \\caption{\\scriptsize{Power off-takes energy sources (propulsion and APU)}}" << std::endl
                               << "     \\end{figure}" << std::endl
                               << std::endl
                               << "\\scriptsize{APU is sized (only) for ground operations at gate (not visible in this mission).}" << std::endl
                               << std::endl
                               << "\\end{frame}" << std::endl
                               << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                               << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                               << std::endl
                               << "\\begin{frame}" << std::endl
                               << " \\frametitle{Energy Conductors and Accumulated Off-takes}" << std::endl
                               << "     \\begin{figure}" << std::endl
                               << "         \\tiny" << std::endl
                               << "         \\def\\svgwidth{10cm}" << std::endl
                               << "         \\import{" + win2lin(relativePath(rtIO_->getPlotDir(), rtIO_->getReportDir())) + "}{"
                                                       + data_->configuration.theProgramName + "_missionPower_EnergyConductors_plot.pdf_tex}" << std::endl
                               << "         \\caption{\\scriptsize{Power off-takes energy conductors and sinks (accumulated)}}" << std::endl
                               << "     \\end{figure}" << std::endl
                               << std::endl
                               << "\\end{frame}" << std::endl
                               << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                               << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl;
    if (data_->configuration.designCase) {
        report_.texReportStream() << "\\begin{frame}" << std::endl
                                   << " \\frametitle{Maximum Energy}" << std::endl
                                   << "     \\begin{figure}" << std::endl
                                   << "         \\tiny" << std::endl
                                   << "         \\def\\svgwidth{10cm}" << std::endl
                                   << "         \\import{" + win2lin(relativePath(rtIO_->getPlotDir(), rtIO_->getReportDir()))
                                                           + "}{" + data_->configuration.theProgramName + "_EnergySinksMaxEnergy_plot.pdf_tex}" << std::endl
                                   << "         \\caption{\\scriptsize{Maximum energy ATA systems}}" << std::endl
                                   << "     \\end{figure}" << std::endl
                                   << std::endl
                                   << "\\end{frame}" << std::endl
                                   << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
                                   << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl;
    }
}
